/**
 * @file Ce fichier gère les requêtes LDAP de type données ; liste des groupe d'un individu, liste des membres d'un groupe... A ne pas confondre avec ldap_auth qui lui gère l'authentification.
 * @author hawkspar
*/

var ensureLoggedin =  require('connect-ensure-login').ensureLoggedIn;
var ldapescape = require("ldap-escape");
var ldap = require('ldapjs');

var client = ldap.createClient({ url: "ldap://frankiz.eleves.polytechnique.fr", timeout: 10000, idleTimeout: 10000});

/**
 * @summary Fonction qui retrouve les groupes du LDAP dont un individu est membre (pas de bind nécessaire)
 * @arg {int} uid - Identifiant de l'individu à interroger
 * @return {string} Liste des uid de groupes (noms flat des groupes) où l'id fourni
 */
function listGroups(uid) {
    return new Promise(function(resolve, reject) {
        var groupsList=[];
        
        client.search("ou=groups,dc=frankiz,dc=net", {scope: "sub", attributes: "uid", filter: ldapescape.filter("(restrictedMemberUid=${id})", {id: uid})}, function(err, res) {
            if (err) {
                reject(err);
            } else {
                res.on('searchEntry', function(entry) { groupsList.push(entry.object.uid); });
                res.on('end', function(res) { resolve(groupsList); });
            }
        });
    });
}
 
/**
 * TBM
 * @summary Fonction qui retrouve la liste des membres d'un binet sur le LDAP
 * @arg {int} uid - Identifiant du groupe à interrogeant (pour des raisons d'identification)
 * @return {string} Liste des uid de groupes où l'id fournie est membre (noms flat des groupes)
 */
function listMembers(uid) {
    client.bind("uid=${uid},ou=eleves,dc=frankiz,dc=net", (err) => { console.log(err); });
    
    return new Promise(function(resolve, reject) {
        var groupsList=[];
        
        client.search("ou=groups,dc=frankiz,dc=net", {scope: "sub", attributes: "uid", filter: ldapescape.filter("(restrictedMemberUid=${id})", {id: uid})}, function(err, res) {
            if (err) {
                reject(err);
            } else {
                res.on('searchEntry', function(entry) { groupsList.push(entry.object.uid); });
                res.on('end', function(res) { resolve(groupsList); });
            }
        });
    });
}

// Synthaxe d'utilisation
listGroups("quentin.chevalier","Ie42fds'eaJm1").then((grList) => { console.log(grList); });